﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DragAndRelateTestApp {
	public class MatrixHandler {

		public MatrixHandler() {
			SetApplication();
			CreateDARMatrix();
			AppendDragAndRelateMenus();
		}

		private SAPbouiCOM.Application mySBOApplication;
		private SAPbouiCOM.Form mySBOMainMenuForm;
		private SAPbouiCOM.Matrix myDarMatrix;
		private const string BXDARMENUMATRIXID = "BXDARMNU";
		private const string BXDARHEADCOLID = "BXHDCOL";
		private const string BXDARNAMECOLID = "BXNMCOL";
		private const string BXDARCOUNTCOLID = "BXCNTCOL";
		private const string DARID_BoMTree = "PPSBoMTree";
		private const string DARID_BoMTreeWOOper = "PPSBoMTWOO";
		private const string DARID_BoMUsage = "PPSBoMUsage";
		private const string DARID_ProdMatReq = "PPSMaW";
		private const string DARID_OpSeqDiagram = "PPSSeqD";
		private const string DARID_JobReqReport = "PPSJobR";
		private const string DARID_AllocStatus = "PPSAllS";
		private const string DARID_MatAccGrid = "PPSMatG";
		private const string DARID_MatAccGraph = "PPSMatH";
		private void CreateDARMatrix() {
			try {

				mySBOMainMenuForm = mySBOApplication.Forms.GetForm("169", 1);

				mySBOMainMenuForm.Freeze(true);
				SAPbouiCOM.Item ourDARMatrix = ConstructDARMenuMatrix();
				ResizeDARMenuMatrix();
			} finally {
				mySBOMainMenuForm.Freeze(false);
				RedrawScreen();
				SAPbouiCOM.Item folderDAR = mySBOMainMenuForm.Items.Item("3");
				folderDAR.Click(0);
			}
		}

		private void AppendDragAndRelateMenus() {
			AddDragAndRelateMenu(DARID_BoMTree, "BoMTree");
			AddDragAndRelateMenu(DARID_BoMTreeWOOper, "BoMTreeWOOperations");
			AddDragAndRelateMenu(DARID_BoMUsage, "BOMUsage");
			AddDragAndRelateMenu(DARID_ProdMatReq, "ProductionMaterialRequirements");
			AddDragAndRelateMenu(DARID_MatAccGrid, "MaterialAccountGrid");
			AddDragAndRelateMenu(DARID_MatAccGraph, "MaterialAccountDiagram");
			AddDragAndRelateMenu(DARID_OpSeqDiagram, "OperationsSequenceDiagram");
			AddDragAndRelateMenu(DARID_JobReqReport, "JobRequirementsReport");
			AddDragAndRelateMenu(DARID_AllocStatus, "AllocationStatus");
		}

		private SAPbouiCOM.Item ConstructDARMenuMatrix() {
			SAPbouiCOM.Item ourDARMatrix = FindItem(BXDARMENUMATRIXID); ;
			if (ourDARMatrix == null) {
				try {
					ourDARMatrix = mySBOMainMenuForm.Items.Add(BXDARMENUMATRIXID, SAPbouiCOM.BoFormItemTypes.it_MATRIX);
					ResizeDARMenuMatrix();
				} catch (Exception e) {
					ourDARMatrix = FindItem(BXDARMENUMATRIXID);
					if (ourDARMatrix == null) {
						throw new Exception(e.Message);
					}

				}

			}
			myDarMatrix = (SAPbouiCOM.Matrix)ourDARMatrix.Specific;
			myDarMatrix.Layout = SAPbouiCOM.BoMatrixLayoutType.mlt_Normal;
			SAPbouiCOM.Column headerCol = CreateMatrixColumnIfDoesNotExist(myDarMatrix, BXDARHEADCOLID, SAPbouiCOM.BoFormItemTypes.it_EDIT);
			headerCol.Visible = false;
			headerCol.Editable = false;
			if (!LoadedFromSRF) {
				CreateUserDataSource(BXDARHEADCOLID, SAPbouiCOM.BoDataType.dt_SHORT_TEXT, 100, false);
				headerCol.DataBind.SetBound(true, null, BXDARHEADCOLID);
			}
			SAPbouiCOM.Column countCol = CreateMatrixColumnIfDoesNotExist(myDarMatrix, BXDARCOUNTCOLID, SAPbouiCOM.BoFormItemTypes.it_EDIT);
			countCol.Visible = false;
			countCol.Editable = false;
			if (!LoadedFromSRF) {
				CreateUserDataSource(BXDARCOUNTCOLID, SAPbouiCOM.BoDataType.dt_LONG_NUMBER, 20, false);
				countCol.DataBind.SetBound(true, null, BXDARCOUNTCOLID);
			}
			SAPbouiCOM.Column nameCol = CreateMatrixColumnIfDoesNotExist(myDarMatrix, BXDARNAMECOLID, SAPbouiCOM.BoFormItemTypes.it_EDIT);
			nameCol.Editable = true;
			nameCol.Width = ourDARMatrix.Width - 20;
			nameCol.TitleObject.Caption = "Boyum DAR Matrix";
			if (!LoadedFromSRF) {
				CreateUserDataSource(BXDARNAMECOLID, SAPbouiCOM.BoDataType.dt_SHORT_TEXT, 100, false);
				nameCol.DataBind.SetBound(true, null, BXDARNAMECOLID);
			}
			myDarMatrix.SelectionMode = SAPbouiCOM.BoMatrixSelect.ms_Single;
			nameCol.Editable = false;
			return ourDARMatrix;
		}

		private void ResizeDARMenuMatrix() {
			SAPbouiCOM.Item sboDARMatrix = (SAPbouiCOM.Item)mySBOMainMenuForm.Items.Item("9");
			int originalSBOMatrixHeight = SboModulesMatrix.Height;
			SAPbouiCOM.Item ourDARMatrix = FindItem(BXDARMENUMATRIXID);
			if (ourDARMatrix != null) {
				sboDARMatrix.Height = originalSBOMatrixHeight / 2;
				ourDARMatrix.Left = sboDARMatrix.Left;
				ourDARMatrix.Top = sboDARMatrix.Top + sboDARMatrix.Height + 1;
				ourDARMatrix.Width = sboDARMatrix.Width;
				ourDARMatrix.Height = sboDARMatrix.Height;
				ourDARMatrix.ToPane = sboDARMatrix.ToPane;
				ourDARMatrix.FromPane = sboDARMatrix.FromPane;
				SAPbouiCOM.Column nameCol = GetMatrixColumnByID(myDarMatrix, BXDARNAMECOLID);
				if (nameCol != null) nameCol.Width = ourDARMatrix.Width - 20;
			}
		}

		private void AddDragAndRelateMenu(string id, string title) {
			bool foundItem = false;
			bool foundSeparator = false;
			SAPbouiCOM.UserDataSource udsDarHead = FindUserDataSource(BXDARHEADCOLID);
			SAPbouiCOM.UserDataSource udsDarName = FindUserDataSource(BXDARNAMECOLID);
			SAPbouiCOM.UserDataSource udsDarCount = FindUserDataSource(BXDARCOUNTCOLID);
			const string LEADINGSPACES = ".     ";
			for (int i = 1; i <= myDarMatrix.RowCount; i++) {
				myDarMatrix.GetLineData(i);
				if (udsDarHead.ValueEx == LEADINGSPACES + "Boyum Test Addon") {
					foundSeparator = true;
				}
				if (udsDarHead.ValueEx == id) {
					udsDarCount.ValueEx = Convert.ToString(Convert.ToInt32(udsDarCount.ValueEx) + 1);
					foundItem = true;
					break;
				}
			}
			if (!foundItem) {
				if (!foundSeparator) {
					udsDarHead.ValueEx = LEADINGSPACES + "Boyum Test Addon";
					udsDarName.ValueEx = LEADINGSPACES + "Boyum Test Addon";
					udsDarCount.ValueEx = "1";
					myDarMatrix.AddRow(1, myDarMatrix.RowCount + 1);
					SetMatrixRowColor(myDarMatrix, myDarMatrix.RowCount);
				}
				udsDarHead.ValueEx = id;
				udsDarName.ValueEx = LEADINGSPACES + "-" + title;
				udsDarCount.ValueEx = "1";
				myDarMatrix.AddRow(1, myDarMatrix.RowCount + 1);
				SetMatrixRowColor(myDarMatrix, myDarMatrix.RowCount);
			}
		}

		private void SetMatrixRowColor(SAPbouiCOM.Matrix matrix, int row) {
			SAPbouiCOM.CommonSetting setting = matrix.CommonSetting;
			System.Drawing.Color newRGBColor = System.Drawing.Color.Black;
			int rowForeColor = newRGBColor.R | (newRGBColor.G << 8) | (newRGBColor.B << 16);
			setting.SetRowFontColor(row, rowForeColor);
		}

		public void RedrawScreen() { try { mySBOMainMenuForm.Update(); } catch { } }

		private SAPbouiCOM.Item FindItem(string uid) {
			try {
				if (mySBOMainMenuForm.Items.Count > 0) return mySBOMainMenuForm.Items.Item(uid);
				else return null;
			} catch (Exception) { return null; }
		}

		private SAPbouiCOM.UserDataSource FindUserDataSource(string uid) {
			try {
				if (SBOFormUserDataSources.Count > 0) return SBOFormUserDataSources.Item(uid);
				else return null;
			} catch (Exception) { return null; }
		}

		private SAPbouiCOM.UserDataSource CreateUserDataSource(string udsID, SAPbouiCOM.BoDataType dataType, int length, bool throwOnException) {
			try {
				return SBOFormUserDataSources.Add(udsID, dataType, length);
			} catch (Exception e) {
				if (throwOnException) throw new Exception("UserDataSource " + udsID + " already exists.", e);
				else return null;
			}
		}

		private SAPbouiCOM.Column CreateMatrixColumnIfDoesNotExist(SAPbouiCOM.Matrix matrix, string columnID, SAPbouiCOM.BoFormItemTypes type) {
			SAPbouiCOM.Column col = GetMatrixColumnByID(matrix, columnID);
			if (col == null) col = matrix.Columns.Add(columnID, type);
			return col;
		}
		private SAPbouiCOM.Column GetMatrixColumnByID(SAPbouiCOM.Matrix matrix, string columnID) {
			try { return matrix.Columns.Item(columnID); } catch (Exception) { return null; }
		}

		private SAPbouiCOM.Item SboDragAndRelateMatrix { get { return mySBOMainMenuForm.Items.Item("9"); } }
		private SAPbouiCOM.Item SboModulesMatrix { get { return mySBOMainMenuForm.Items.Item("6"); } }

		private SAPbouiCOM.DataSource mySBOFormDataSources;
		private SAPbouiCOM.DataSource SBOFormDataSources {
			get {
				if (mySBOFormDataSources == null) mySBOFormDataSources = mySBOMainMenuForm.DataSources;
				return mySBOFormDataSources;
			}
		}

		private SAPbouiCOM.UserDataSources mySBOFormUserDataSources;
		private SAPbouiCOM.UserDataSources SBOFormUserDataSources {
			get {
				if (mySBOFormUserDataSources == null && SBOFormDataSources != null) mySBOFormUserDataSources = SBOFormDataSources.UserDataSources;
				return mySBOFormUserDataSources;
			}
		}

		private bool LoadedFromSRF { get { return false; } }

		private void SetApplication() {
			SAPbouiCOM.SboGuiApi sboGuiApi = new SAPbouiCOM.SboGuiApi();
			string connectionString = "0030002C0030002C00530041005000420044005F00440061007400650076002C0050004C006F006D0056004900490056";
			sboGuiApi.Connect(connectionString);
			mySBOApplication = sboGuiApi.GetApplication(-1);
			mySBOApplication.ItemEvent += new SAPbouiCOM._IApplicationEvents_ItemEventEventHandler(SBO_Application_ItemEvent);
		}

		private void SBO_Application_ItemEvent(string formUID, ref SAPbouiCOM.ItemEvent pVal, out bool bubbleEvent) {
			bubbleEvent = true;
			if (pVal.FormType == 169) {
				SAPbouiCOM.BoEventTypes eventEnum = pVal.EventType;
				switch (eventEnum) {
					case SAPbouiCOM.BoEventTypes.et_FORM_RESIZE:
						if (!pVal.BeforeAction) {
							if (myDarMatrix != null) {
								SAPbouiCOM.Item mm = SboModulesMatrix;
								SAPbouiCOM.Item dm = SboDragAndRelateMatrix;
								dm.Width = mm.Width;
								dm.Height = mm.Height;
								ResizeDARMenuMatrix();
							}
						}
						break;
				}
			}
		}
	}
}
